#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ascii_paradise/animation/display.rb'
# =========================================================================== #
require 'ascii_paradise/base/base.rb'
begin
  require 'colours/rainbow_colours/returnln_plain.rb'
rescue LoadError; end # Silent rescue.

module AsciiParadise

class Animation < Base # === AsciiParadise::Animation

  # ========================================================================= #
  # === display
  #
  # This method can display a frame of an animated ASCII class.
  #
  # It is essentially just an output-method. The method will also
  # optionally allow the user to use a specific colour. This
  # colour must exist as part of the Colours namespace.
  # ========================================================================= #
  def display(frame)
    i = frame.dup # Must work on a copy.
    if @use_colours # Only do any of the following if we do happen to use colours.
      if @use_rainbow_colours
        i = ::Colours::RainbowColours.returnln_plain(i)
      elsif @use_this_colour and
         ::Colours.respond_to?(@use_this_colour.to_sym)
        i = ::Colours.send(@use_this_colour, i)
      end
    end
    e i # Finally display the frame here.
  end; alias display_this_frame display # === display_this_frame

  # ========================================================================= #
  # === display_the_frame
  #
  # This method will display the individual "frame" of an animated ASCII
  # component.
  # ========================================================================= #
  def display_the_frame(
      frame = @frame
    )
    # ======================================================================= #
    # Since since as of January 2020 we will also show the name of the
    # file at hand.
    # ======================================================================= #
    if debug?
      e steelblue('Filename: ')+sfile(guess_filename)
    end
    # ======================================================================= #
    # Display the particular frame at hand next:
    # ======================================================================= #
    display frame
    # ======================================================================= #
    # === If the user wants to see red numbers, we show them next
    #
    # The idea behind these red numbers is to make it easier to show
    # which frame may be mis-aligned and so forth.
    # ======================================================================= #
    if @wait_for_keypress_on_each_frame == :also_show_red_numbers
      e
      1.upto(9) {|number| print swarn(number) }
      1.upto(9) {|number| print swarn(number) }
      1.upto(9) {|number| print swarn(number) }
      e
    end
    # ======================================================================= #
    # === Show the frame number if we are running in debug mode
    # ======================================================================= #
    if debug?
      e
      e steelblue('Frame number: ')+
        royalblue((@index_position+1).to_s)+
        mediumpurple(
          ' ('+return_filename(@index_position+1)+')'
        )
      e # Display a newline too.
    end
  end

end; end