#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === AsciiParadise::AnimatedPacman
#
# This animated ASCII component will show pacman running in a
# never-ending loop.
# =========================================================================== #
require 'ascii_paradise/animation/animation.rb'

module AsciiParadise

class AnimatedPacman < Animation

  include Math # So we can write .atan2() and similar.

  # ========================================================================= #
  # === YELLOW_AND_RESET
  # ========================================================================= #
  YELLOW_AND_RESET = "\033[1;33m\033[?25l"

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    super()
    run if run_already
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    print CLEAR_COMMAND+YELLOW_AND_RESET # This will also turn the stuff to yellow.
    z  = 0.0
    dz = 0.05
    loop {
      z +=  dz # Add the 0.05 difference to z.
      dz = -dz if ( z < 0.0 || z > 0.2)
      print "\033[0;0f"
      (-10..10).each {|y|
        s = ''.dup # This will be the string that will be displayed.
        (-39..39).each {|x|
          s << ((x/2) ** 2 + y ** 2 < 11 ** 2\
          && atan2(y,x/2) <  PI * (1.0 - z) \
          && atan2(y,x/2) > -PI * (1.0 - z) ? '@':' ')
        }
        puts s
      }
      sleep 0.03
    } # ';reset # Get cursor back :)
  end

end

  # ========================================================================= #
  # === AsciiParadise.animated_pacman
  # ========================================================================= #
  def self.animated_pacman
    AsciiParadise::AnimatedPacman.new
  end

  # ========================================================================= #
  # === AsciiParadise.animatedpacman
  # ========================================================================= #
  def self.animatedpacman
    AsciiParadise::AnimatedPacman
  end

end

if __FILE__ == $PROGRAM_NAME
  AsciiParadise::AnimatedPacman.new
end # animated_pacman