#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === AsciiParadise::AnimatedProgressBar
#
# Usage:
# 
#   You can update the progress and render independently if you wish.
#
# Use
#
#   ProgessBar#progress= for that and don't pass a value to #update.
# =========================================================================== #
require 'ascii_paradise/animation/animation.rb'

module AsciiParadise

class AnimatedProgressBar < Animation

  # ========================================================================= #
  # The various constants - top left, top, top right and so forth.
  #
  # These may be Unicode symbols.
  # ========================================================================= #
  TopLeft      = "\342\224\214"
  Top          = "\342\224\200"
  TopRight     = "\342\224\220"
  Left         = "\342\224\202"
  Right        = "\342\224\202"
  BottomLeft   = "\342\224\224"
  Bottom       = "\342\224\200"
  BottomRight  = "\342\224\230"
  LightCheck   = "\e[32m\342\234\223\e[0m"
  Check        = "\e[32m\342\234\224\e[0m" # A checkbox here.
  # ========================================================================= #
  # === BACKTRACKING
  # ========================================================================= #
  BACKTRACKING = "\e[3A"
  MAX_VALUE    = 50
  STEP_TO      = 100
  STEP_SIZE    = 1.0

  attr_accessor :progress

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      pad = ['','',''],
      io  = $stdout
    )
    register_sigint
    reset
    @pad = pad # An Array. Initially empty.
    @io  = io # Stdout.
    # output_three_newlines
    run
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    @progress = 0
  end

  # ========================================================================= #
  # === output_three_newlines
  # ========================================================================= #
  def output_three_newlines
    puts N+N+N # Make three newlines. Unsure why.
  end

  # ========================================================================= #
  # === update
  #
  # The main method that will output the bar.
  # ========================================================================= #
  def update(progress = nil)
    @progress = progress if progress
    prog   = @progress
    filled = (0.5 * prog).round
    free   = MAX_VALUE - filled
    # ======================================================================= #
    # Next draw the box.
    # ======================================================================= #
    @io.print BACKTRACKING,
      @pad[0], TopLeft, Top * MAX_VALUE, TopRight, "\n",
      @pad[1], Left, "\e[44m", ' ' * filled, "\e[46m", ' ' * free, 
      "\e[0m", Right, ' ', prog.round, "%\n",
      @pad[2], BottomLeft, Bottom * MAX_VALUE, BottomRight, N
    @io.flush
  end

  # ========================================================================= #
  # === finish
  # ========================================================================= #
  def finish
    @io.print BACKTRACKING,
      @pad[0], TopLeft, Top * MAX_VALUE, TopRight, "\n",
      @pad[1], Left, "\e[44m", " " * MAX_VALUE, "\e[0m", Right, " ", Check, "    \n",
      @pad[2], BottomLeft, Bottom * MAX_VALUE, BottomRight, "\n"
    @io.flush
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    0.step(STEP_TO, STEP_SIZE) { |i|
      update(i)
      sleep(rand * 1)
    }
    finish
  end

end

  # ========================================================================= #
  # === AsciiParadise.animated_progress_bar
  # ========================================================================= #
  def self.animated_progress_bar
    AsciiParadise::AnimatedProgressBar.new
  end

  # ========================================================================= #
  # === AsciiParadise.animatedprogressbar
  # ========================================================================= #
  def self.animatedprogressbar
    AsciiParadise::AnimatedProgressBar
  end

end

if __FILE__ == $PROGRAM_NAME
  AsciiParadise::AnimatedProgressBar.new
end # progressbar