#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === AsciiParadise::Counter
#
# Usage example for this class here:
#
#   AsciiParadise::Counter.new(20, 5)
#
# =========================================================================== #
require 'ascii_paradise/animation/animation.rb'

module AsciiParadise

class Counter < Animation


  # ========================================================================= #
  # === N_PADDING
  # ========================================================================= #
  N_PADDING = 3

  # ========================================================================= #
  # === DEFAULT_COUNTDOWN_FROM
  # ========================================================================= #
  DEFAULT_COUNTDOWN_FROM = 25

  # ========================================================================= #
  # === BROWN
  # ========================================================================= #
  BROWN = Colours::BROWN

  # ========================================================================= #
  # === initialize
  #
  # The second argument is the default padding to use.
  # ========================================================================= #
  def initialize(
      countdown_from   = nil,
      use_this_padding = N_PADDING,
      run_already      = true
    )
    reset
    set_countdown_from(countdown_from)
    set_padding_to_use(use_this_padding)
    case countdown_from
    when :dont_run_yet
      run_already = false
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    @colour_to_use = BROWN
  end

  # ========================================================================= #
  # === set_countdown_from
  # ========================================================================= #
  def set_countdown_from(i = nil)
    i = DEFAULT_COUNTDOWN_FROM if i.is_a? Symbol
    i = DEFAULT_COUNTDOWN_FROM if i.nil?
    @countdown_from = i.to_i # Must be Integer.
  end; alias start_from  set_countdown_from # === start_from
       alias start_from= set_countdown_from # === start_from=

  # ========================================================================= #
  # === Counter[]
  # ========================================================================= #
  def self.[](i)
   ascii_counter = Counter.new(:dont_run_yet)
   ascii_counter.start_from = i
   ascii_counter.run
  end

  # ========================================================================= #
  # === count_down
  #
  # Use this method to count down. We start at 25 for now.
  # ========================================================================= #
  def count_down(i = @countdown_from)
    i.downto(0).each { |x|
      _ = ( ('%'+@padding_to_use.to_s+'s') % x.to_s)
      print @colour_to_use+R+_
      STDOUT.flush
      begin
        sleep 1
      rescue Interrupt
        puts; exit
      end # Silent rescue.
    }
  end

  # ========================================================================= #
  # === set_padding_to_use  
  # ========================================================================= #
  def set_padding_to_use(i)
    i = i.size if i.is_a? String
    i = i.to_i
    @padding_to_use = i
  end

  # ========================================================================= #
  # === run                                                          (run tag)
  # ========================================================================= #
  def run
    count_down
  end
  
end

  AsciiCounter = Counter

  # ========================================================================= #
  # === AsciiParadise.ascii_counter
  # ========================================================================= #
  def self.ascii_counter(count_down_from = 50)
    AsciiParadise::Counter.new(count_down_from)
  end

  # ========================================================================= #
  # === AsciiParadise.Counter
  # ========================================================================= #
  def self.Counter
    AsciiParadise::Counter
  end

end

if __FILE__ == $PROGRAM_NAME
  AsciiParadise::Counter.new(ARGV.first)
end # ascii_counter 33