#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === AsciiParadise::ProgressBar
#
# This class will display an ASCII progress-bar.
# =========================================================================== #
require 'ascii_paradise/animation/animation.rb'

module AsciiParadise

class ProgressBar < Animation

  # ========================================================================= #
  # === INTERVAL
  #
  # How long we will wait. The larger, the slower the progress-bar will
  # appear.
  # ========================================================================= #
  INTERVAL = 0.2

  # ========================================================================= #
  # === START
  # ========================================================================= #
  START = "\r["

  # ========================================================================= #
  # The length of the AsciiBar comes next. The higher, the longer the
  # bar.
  # ========================================================================= #
  LENGTH = N_TIMES = 50

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    register_sigint
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    @length = LENGTH
  end

  # ========================================================================= #
  # === run_the_progress_bar
  # ========================================================================= #
  def run_the_progress_bar
    @length.times {|i|
      output_string = START.dup # Need .dup here because otherwise the START-string will be appended.
      @length.times {|x|
        # =================================================================== #
        # Next we build up our output string. This string will fluctate
        # at every interval.
        # =================================================================== #
        output_string << (
          x == i && ( i % 2 == 0 && 'C'||'c')|| (x < i && '-' || ( x % 2 == 0 && ' '|| 'o' ))
        )
      }
      print output_string+']'
      STDOUT.flush
      sleep INTERVAL
    }
    e
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    run_the_progress_bar
  end

end

  # ========================================================================= #
  # === AsciiParadise.progress_bar
  #
  # You can also invoke this method via:
  #
  #   AsciiParadise.progressbar
  #
  # ========================================================================= #
  def self.progress_bar
    ProgressBar.new
  end; self.instance_eval { alias ascii_progress_bar progress_bar } # === AsciiParadise.ascii_progress_bar
       self.instance_eval { alias progressbar        progress_bar } # === AsciiParadise.progressbar

  # ========================================================================= #
  # === AsciiParadise.ProgressBar
  # ========================================================================= #
  def self.ProgressBar
    ProgressBar
  end

end

if __FILE__ == $PROGRAM_NAME
  AsciiParadise::ProgressBar.new
end # progress_bar