#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
require 'ascii_paradise/animation/animation.rb'

module AsciiParadise # === AsciiParadise::RotatingSlash

class RotatingSlash < Animation

  # ========================================================================= #
  # === DEFAULT_DELAY
  # ========================================================================= #
  DEFAULT_DELAY = 0.15

  # ========================================================================= #
  # === ARRAY
  #
  # This constant is an Array that keeps the different tokens which will
  # be displayed on the commandline.
  #
  # Note that the last character in that array, the '/', will be the
  # token that we will keep displayed.
  #
  # There are six entries in this Array.
  # ========================================================================= #
  ARRAY = %w(
    | / - \\ '  /'
  )#1 2 3 4  5  6

  # ========================================================================= #
  # === BACKSPACE
  # ========================================================================= #
  BACKSPACE = "\b" # \b is Backspace.

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    register_sigint
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    @delay = DEFAULT_DELAY # We use this default.
  end

  # ========================================================================= #
  # === can_we_finish?
  # ========================================================================= #
  def can_we_finish?
    rand(300) == 0
  end

  # ========================================================================= #
  # === finish
  # ========================================================================= #
  def show_finish_message # finish and end here.
    puts 'Finished!'
  end

  # ========================================================================= #
  # === delay?
  # ========================================================================= #
  def delay?
    @delay
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    # ======================================================================= #
    # The tokens are those that will appear to "dance".
    # ======================================================================= #
    $stdout.sync = true # must be true else we won't see a thing.
    loop {
      print ARRAY.unshift(ARRAY.pop).last
      sleep delay?
      print BACKSPACE
      if can_we_finish? # Randomly finish here, via a message.
        show_finish_message
        break
      end
    }
  end

end

  # ========================================================================= #
  # === AsciiParadise.rotating_slash
  # ========================================================================= #
  def self.rotating_slash
    AsciiParadise::RotatingSlash.new
  end

  # ========================================================================= #
  # === AsciiParadise.RotatingSlash
  # ========================================================================= #
  def self.RotatingSlash
    AsciiParadise::RotatingSlash
  end

end

if __FILE__ == $PROGRAM_NAME
  AsciiParadise::RotatingSlash.new
end # rotating_slash