#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ascii_paradise/ascii_say/say.rb'
# =========================================================================== #
module AsciiParadise

  require 'ascii_paradise/ascii_say/parse_static_ascii.rb'
  require 'ascii_paradise/toplevel_methods/available_ascii_components.rb'

  # ========================================================================= #
  # === AsciiParadise.ascii_say
  #
  # Use a simplified ascii-talker, a bit like in a cartoon.
  #
  # The second argument of this method tells us which subcomponent 
  # is to be used.
  #
  # Usage examples:
  #
  #   AsciiParadise.say 'Yo there how do you do?', :wizard
  #
  # ========================================================================= #
  def self.ascii_say(
      i                      = 'Hello world!',
      use_which_subcomponent = :random
    )
    if i.is_a? Array
      i = i.join(' ')
    end
    case use_which_subcomponent.to_sym
    # ======================================================================= #
    # === :random
    #
    # This entry point will list all possible components that respond
    # to .say().
    # ======================================================================= #
    when :random
      array = [
        AsciiParadise::Alien,
        AsciiParadise::Buddha,
        AsciiParadise::CatInTheHat,
        AsciiParadise::DomeTent,
        AsciiParadise::FlowerDecoration,
        AsciiParadise::Wizard,
        AsciiParadise::Knight,
        AsciiParadise::SexyAngel,
        AsciiParadise::Cowsay,
        AsciiParadise::Ponysay
      ]
      subcomponent_to_use = array.sample
    # ======================================================================= #
    # === :alien
    # ======================================================================= #
    when :alien
      subcomponent_to_use = AsciiParadise::Alien
    # ======================================================================= #
    # === :buddha
    # ======================================================================= #
    when :buddha
      subcomponent_to_use = AsciiParadise::Buddha
    # ======================================================================= #
    # === :dome_tent
    # ======================================================================= #
    when :dome_tent,
         :tent
      subcomponent_to_use = AsciiParadise::DomeTent
    # ======================================================================= #
    # === :flower
    # ======================================================================= #
    when :flower,
         :flower_decoration
      subcomponent_to_use = AsciiParadise::FlowerDecoration
    # ======================================================================= #
    # === :cat
    # ======================================================================= #
    when :cat,
         :cat_in_the_hat
      subcomponent_to_use = AsciiParadise::CatInTheHat
    # ======================================================================= #
    # === :wizard
    # ======================================================================= #
    when :wizard
      subcomponent_to_use = AsciiParadise::Wizard
    # ======================================================================= #
    # === :knight
    # ======================================================================= #
    when :knight
      subcomponent_to_use = AsciiParadise::Knight
    # ======================================================================= #
    # === :sexy_angel
    # ======================================================================= #
    when :sexy_angel
      subcomponent_to_use = AsciiParadise::SexyAngel
    # ======================================================================= #
    # === :cowsay
    # ======================================================================= #
    when :cowsay,
         :cow
      subcomponent_to_use = AsciiParadise::Cowsay
    # ======================================================================= #
    # === :ponysay
    # ======================================================================= #
    when :ponysay,
         :pony
      subcomponent_to_use = AsciiParadise::Ponysay
    end
    subcomponent_to_use.say i
  end; self.instance_eval { alias bubble_say   ascii_say } # === AsciiParadise.bubble_say
       self.instance_eval { alias bubble_speak ascii_say } # === AsciiParadise.bubble_speak
       self.instance_eval { alias say          ascii_say } # === AsciiParadise.say

  # ========================================================================= #
  # === AsciiParadise.cowsay
  #
  # The ascii cow will say the given text.
  #
  # Invocation example:
  #
  #   AsciiParadise.cowsay 'I am a happy cow!'
  #
  # ========================================================================= #
  def self.cowsay(i = 'Hello world!')
    say(i, :cow)
  end

  # ========================================================================= #
  # === AsciiParadise.ponysay
  #
  # The ascii pony will say the given text.
  #
  # Invocation example:
  #
  #   AsciiParadise.ponysay 'I am a happy pony!'
  #
  # ========================================================================= #
  def self.ponysay(i = 'Hello world!')
    say(i, :pony)
  end

  # ========================================================================= #
  # === AsciiParadise.flower_decoration
  #
  # Invocation example:
  #
  #   AsciiParadise.flower_decoration(("Once upon a time ... \n there were three evil witches."*10))
  #
  # ========================================================================= #
  def self.flower_decoration(i = 'Hello world!')
    say(i, :flower_decoration)
  end

end