#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === AsciiParadise::AsciiTable::Row
#
# This class will handle the row-elements, which are usually '----' tokens.
# =========================================================================== #
require 'ascii_paradise/asciitable/toplevel_methods.rb'

module AsciiParadise

class AsciiTable

class Row

  # ========================================================================= #
  # === initialize
  #
  # Initialize with _width_ and _options_.
  # ========================================================================= #
  def initialize(table, array = [])
    reset
    @table = table
    array.each { |item| self << item }
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    @cells = []
    @cell_index = 0
  end

  # ========================================================================= #
  # === table?
  # ========================================================================= #
  def table?
    @table
  end; alias table table? # === table

  # ========================================================================= #
  # === add_cell
  # ========================================================================= #
  def add_cell(i)
    options = i.is_a?(Hash) ? i : {:value => i}
    cell = Cell.new(options.merge(:index => @cell_index, :table => @table))
    # ======================================================================= #
    # The next variables are Integers.
    # ======================================================================= #
    @cell_index += cell.colspan # Count up the cell index here.
    @cells << cell
  end; alias << add_cell # === <<

  # ========================================================================= #
  # === height
  # ========================================================================= #
  def height
    cells.map { |c| c.lines.count }.max
  end; alias height? height # === height?

  # ========================================================================= #
  # === cells?
  # ========================================================================= #
  def cells?
    @cells
  end; alias cells cells? # === cells

  # ========================================================================= #
  # === render
  # ========================================================================= #
  def render
    # ======================================================================= #
    # Obtain a pointer towards the horizontal character, which is usually
    # '-'.
    # ======================================================================= #
    horizontal_token = @table.style.border_y
    # ======================================================================= #
    # Next we check whether to colourize this.
    # ======================================================================= #
    if AsciiTable.use_colours?
      horizontal_token = Colours.send(
        AsciiParadise::AsciiTable.use_which_colours?, horizontal_token
      )
    end
    array = (0...height).to_a
    array.map! { |line|
      horizontal_token + cells.map { |cell|
        cell.render(line)
      }.join(horizontal_token) + horizontal_token
    }
    result = array.join("\n")
    return result
  end

  # ========================================================================= #
  # === []
  # ========================================================================= #
  def [](i)
    cells[i]
  end

end; end; end