#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === AsciiParadise::Base
#
# The generic base-class for this project. Other classes in this project
# can subclass from it.
# =========================================================================== #
# require 'ascii_paradise/base/base.rb'; < Base
# =========================================================================== #
require 'ascii_paradise/constants/constants.rb'
require 'ascii_paradise/toplevel_methods/clear_screen.rb'
require 'ascii_paradise/toplevel_methods/misc.rb'
require 'ascii_paradise/toplevel_methods/sort_files.rb'
require 'ascii_paradise/base/colours.rb'
require 'ascii_paradise/base/debug.rb'
require 'ascii_paradise/base/initialize.rb'
require 'ascii_paradise/base/menu.rb'
require 'ascii_paradise/base/reset.rb'
require 'ascii_paradise/toplevel_methods/show_available_ascii_components.rb'

module AsciiParadise

class Base # === AsciiParadise::Base

  alias ee print

  # ========================================================================= #
  # === register_sigint
  # ========================================================================= #
  def register_sigint
    ::AsciiParadise.register_sigint
  end

  # ========================================================================= #
  # === sort_files
  # ========================================================================= #
  def sort_files(i)
    ::AsciiParadise.sort_files(i)
  end

  # ========================================================================= #
  # === do_not_use_clear
  # ========================================================================= #
  def do_not_use_clear
    @do_clear = false
  end

  # ========================================================================= #
  # === clear_screen
  # ========================================================================= #
  def clear_screen
    ::AsciiParadise.clear_screen
  end

  # ========================================================================= #
  # === do_wait_for_keypress_event
  #
  # The argument can be true, by default, or also another argument. Some
  # symbols may have a special instruction.
  # ========================================================================= #
  def do_wait_for_keypress_event(i = true)
    @wait_for_keypress_on_each_frame = i
  end

  # ========================================================================= #
  # === do_not_run_already
  # ========================================================================= #
  def do_not_run_already
    @run_already = false
  end; alias do_not_run_yet do_not_run_already # === do_not_run_yet

  # ========================================================================= #
  # === static_dir?
  # ========================================================================= #
  def static_dir?
    "#{project_base_dir?}static_ascii/"
  end

  # ========================================================================= #
  # === project_base_dir?
  # ========================================================================= #
  def project_base_dir?
    ::AsciiParadise.project_base_dir?
  end

  # ========================================================================= #
  # === show_available_components
  # ========================================================================= #
  def show_available_components
    ::AsciiParadise.show_available_ascii_components
  end

  # ========================================================================= #
  # === report_how_many_animated_components_exist
  # ========================================================================= #
  def report_how_many_animated_components_exist
    e 'There are '+
      sfancy(::AsciiParadise.n_animated_components?.to_s)+
      ' animated entries available.'
  end

  # ========================================================================= #
  # === is_animated?
  #
  # By default, sublcasses of this class are not animated. This is different
  # for those ruby files that subclass from AsciiParadise::Animation.
  # ========================================================================= #
  def is_animated?
    false
  end

  # ========================================================================= #
  # === show_help                                                  (help tag)
  #
  # To invoke this method, try:
  #
  #   fireworks --help
  #
  # ========================================================================= #
  def show_help
    ::AsciiParadise.show_help
  end; alias help show_help # === help

  # ========================================================================= #
  # === AsciiParadise::Base.animation_dir?
  # ========================================================================= #
  def self.animation_dir?
    ::AsciiParadise.animations_dir?
  end

  # ========================================================================= #
  # === animation_directory?
  # ========================================================================= #
  def animation_directory?
    Base.animation_dir?
  end; alias animation_dir? animation_directory? # === animation_dir?
       alias which_dir?     animation_directory? # === which_dir?
       alias main_dir?      animation_directory? # === main_dir?

  # ========================================================================= #
  # === return_basename_of_this_file_without_the_extension
  # ========================================================================= #
  def return_basename_of_this_file_without_the_extension(i)
    File.basename(i).sub(/#{File.extname(i)}$/,'')
  end

  # ========================================================================= #
  # === AsciiParadise::Base.run
  # ========================================================================= #
  def self.run
    new
  end

end; end

# =========================================================================== #
# Code to mini-test this class.
# =========================================================================== #
if __FILE__ == $PROGRAM_NAME
  require 'colours/autoinclude'
  e 'The animation directory can be found here:'
  e
  e '  '+sfancy(AsciiParadise::Base.new.animation_directory?)
  e
  e 'The template directory can be found here:'
  e
  e '  '+sfancy(AsciiParadise.template_directory?)
  e
end