#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ascii_paradise/base/colours.rb'
# =========================================================================== #
require 'ascii_paradise/colours/colours.rb'
require 'ascii_paradise/toplevel_methods/colour_parse_this_string.rb'

module AsciiParadise

class Base # === AsciiParadise::Base

  # ========================================================================= #
  # === set_use_this_colour
  #
  # Set a specific colour through this method here.
  #
  # Setting a random colour is also possible, e. g. by using
  # as argument :random_colour.
  # ========================================================================= #
  def set_use_this_colour(
      i = ::AsciiParadise.colour?
    )
    case i
    when nil, :default
      i = DEFAULT_COLOUR_TO_USE # Hardcoded default in this case.
    end
    i = i.to_s.dup # Do not retain the String as frozen.
    i.delete!('-') if i.include? '--'
    case i # case tag
    when /^-?-?random(_|-)?colour$/i # handofgod --colour=random-colour
      i = return_random_colour
    end
    i = i.to_sym unless i.is_a? Symbol
    @use_this_colour = i
  end; alias set_colour      set_use_this_colour # === set_colour
       alias use_this_colour set_use_this_colour # === use_this_colour

  # ========================================================================= #
  # === do_use_random_colour
  # ========================================================================= #
  def do_use_random_colour
    set_colour(:random_colour)
  end; alias use_a_random_colour do_use_random_colour # === use_a_random_colour

  # ========================================================================= #
  # === return_random_colour
  #
  # This method will return, as String, a random HTML colour, such
  # as "steelblue".
  # ========================================================================= #
  def return_random_colour
    ::Colours.random_html_colour
  end

  # ========================================================================= #
  # === simp
  # ========================================================================= #
  def simp(i)
    if @use_colours
      AsciiParadise.simp(i)
    else
      ''
    end
  end

  # ========================================================================= #
  # === sfancy
  # ========================================================================= #
  def sfancy(i)
    if @use_colours
      AsciiParadise.sfancy(i)
    else
      ''
    end
  end

  # ========================================================================= #
  # === sfile
  # ========================================================================= #
  def sfile(i)
    if @use_colours
      AsciiParadise.sfile(i)
    else
      ''
    end
  end

  # ========================================================================= #
  # === swarn
  # ========================================================================= #
  def swarn(i)
    if @use_colours
      AsciiParadise.swarn(i)
    else
      ''
    end
  end

  # ========================================================================= #
  # === Base.e
  # ========================================================================= #
  def self.e(i = '')
    ::AsciiParadise.e(i)
  end

  # ========================================================================= #
  # === e
  # ========================================================================= #
  def e(i = '')
    Base.e(i)
  end

  # ========================================================================= #
  # === remove_trailing_ansci_escape_code
  # ========================================================================= #
  def remove_trailing_ansci_escape_code(i)
    ::Colours.remove_trailing_end_from(i)
  end

  # ========================================================================= #
  # === rev
  # ========================================================================= #
  def rev
    if @use_colours
      ::Colours.rev
    else
      ''
    end
  end

  # ========================================================================= #
  # === colour_parse_this_string
  # ========================================================================= #
  def colour_parse_this_string(i)
    ::AsciiParadise.colour_parse_this_string(i)
  end

  # ========================================================================= #
  # === steelblue
  # ========================================================================= #
  def steelblue(i = '')
    i = i.dup
    i = ::Colours.send(__method__, i) if @use_colours
    i
  end; alias tomato steelblue # === tomato

  # ========================================================================= #
  # === tomato
  # ========================================================================= #
  def tomato(i = '')
    i = i.dup
    i = ::Colours.send(__method__, i) if @use_colours
    i
  end

  # ========================================================================= #
  # === slategrey
  # ========================================================================= #
  def slategrey(i = '')
    i = i.dup
    i = ::Colours.send(__method__, i) if @use_colours
    i
  end

  # ========================================================================= #
  # === royalblue
  # ========================================================================= #
  def royalblue(i = '')
    i = i.dup
    i = ::Colours.send(__method__, i) if @use_colours
    i
  end

end; end