#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ascii_paradise/base/menu.rb'
# =========================================================================== #
module AsciiParadise

class Base

  require 'ascii_paradise/toplevel_methods/available_ascii_components.rb'
  require 'ascii_paradise/toplevel_methods/available_ascii_components_options.rb'
  require 'ascii_paradise/toplevel_methods/n_animated_components.rb'
  require 'ascii_paradise/toplevel_methods/run_this_animated_component.rb'
  require 'ascii_paradise/commandline/help.rb'

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(i)
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i.to_s # case tag
      # ===================================================================== #
      # === volcano --full-debug
      #
      # This entry-point is called "full-debug", which means that we will
      # try to heavily "debug" the ascii-animation at hand, e. g. correct
      # faulty entries and such.
      # ===================================================================== #
      when /^-?-?everything$/i,
           /^-?-?full(-|_)?debug$/i,
           /^-?-?query$/i,
           /^-?-?frame(-|_)?wait$/i,
           /^-?-?all$/i,
           /^-?-?debugeverything$/i,
           /^-?-?debug(-|_)?per(-|_)?frame$/i,
           '-all',
           '-query'
        enable_debug
        do_wait_for_keypress_event
      # ===================================================================== #
      # === turtles --random
      # ===================================================================== #
      when /^-?-?random$/i
        sample = AsciiParadise.available_ascii_components?.sample
        AsciiParadise.run(sample)
        exit
      # ===================================================================== #
      # === turtles --overview
      # ===================================================================== #
      when *AsciiParadise.available_ascii_components_options
        show_available_components
        exit
      # ===================================================================== #
      # === turtles --colourspray
      # ===================================================================== #
      when /^-?-?colour(-|_| )?spray$/i,
           /^-?-?use(-|_| )?colour(-|_| )?spray$/i
        # =================================================================== #
        # Must invoke use_a_random_colour here so that we start by
        # using a random colour already.
        # =================================================================== #
        use_a_random_colour
        do_use_colour_spray
      # ===================================================================== #
      # === ademo --frame
      # ===================================================================== #
      when /^-?-?debug$/,
           /^-?-?frames?$/,
           /^-?-?enable(-|_)?frames$/
        enable_debug
      # ===================================================================== #
      # === volcano --numbers
      # ===================================================================== #
      when /^-?-?numbers\??$/i,
           /^-?-?super(-|_)?debug$/,
           /^-?-?individual(_|-)?frames$/i, # === swimming_lanes --individual-frames
           /^-?-?slow$/i
        enable_debug
        do_wait_for_keypress_event(:also_show_red_numbers) # jumpingpig --numbers
      # ===================================================================== #
      # === turtle --framerate
      # === whip --framerate
      # ===================================================================== #
      when /-?-?framerate\??$/i,
           /-?-?delay\??$/i
        _ = self
        _.do_not_run_yet
        e 'The delay for '+simp(self.class.to_s)+
          ' is '+sfancy(_.delay?.to_s+' seconds')+' per frame.'
        exit
      # ===================================================================== #
      # === volcano --wait-for-keypress
      # ===================================================================== #
      when /^-?-?keypress$/i,
           /^-?-?wait$/i, # diver --wait
           /^-?-?wait(_|-)?for(_|-)?keypress$/i,
           /^-?-?steps?$/i
        do_wait_for_keypress_event
      # ===================================================================== #
      # === volcano --help
      # ===================================================================== #
      when /^-?-?help$/i # help tag
        show_help
        exit
      # ===================================================================== #
      # === volcano --n_animated_components?
      # ===================================================================== #
      when /^-?-?n(_|-)?animated(_|-)?components\??$/i,
           /^-?-?n(_|-)?components\??$/i # === volcano --n_components?
        report_how_many_animated_components_exist
        exit
      # ===================================================================== #
      # === volcano --rainbow-colours
      # === volcano --rainbows
      # ===================================================================== #
      when /^-?-?rainbow(-|_| )?colou?rs$/i,
           /^-?-?rainbow(_|-| )?line$/i,
           /^-?-?rainbows?$/i
        do_use_rainbow_colours
      # ===================================================================== #
      # === volcano --show-frame=25
      # === volcano --frame15
      # ===================================================================== #
      when /^-?-?show(-|_)?frames?=(.+)$/i, # === $2
           /^-?-?frame(\d+)$/i # === $2
        _ = $1.to_s.dup
        _ = $2.to_s.dup if $2
        show_frame_at_this_position(_)
        exit
      # ===================================================================== #
      # === ademon :default
      # ===================================================================== #
      when :default,
           'default'
        set_run_n_times :default
      # ===================================================================== #
      # === turtles --run-n-times=2
      # ===================================================================== #
      when /^-?-?run(-|_)?n(-|_)?times=(\d+)$/i # $3
        set_run_n_times($3.to_s.dup)
      # ===================================================================== #
      # === turtles --half-colour
      # ===================================================================== #
      when /^-?-?half(-|_)?colou?r$/
        do_use_half_colours
      # ===================================================================== #
      # === turtles --disco-inferno
      #
      # URL is at:
      #
      #   http://rubular.com/r/eEYhiWpEpx
      #
      # Additional invocation examples:
      #
      #   trampoline --disco-inferno
      #   volcano --disco-inferno
      #
      # ===================================================================== #
      when /^-?-?disco(-|_)?inferno$/i,
           /colour-?animation/,
           /disco/,
           /inferno/,
           /random-?colours/, # earth --random-colours
           '--random-colours'
        do_use_disco_inferno
      # ===================================================================== #
      # === turtles --is_animated?
      # ===================================================================== #
      when /^-?-?is(-|_)?animated\??$/i,
            /^-?-?animated\??$/i
        e is_animated?
        exit
      # ===================================================================== #
      # === turtles 1
      # ===================================================================== #
      when /^\d+$/ # If only numbers.
        set_run_n_times(i)
      # ===================================================================== #
      # === turtles --random-colour
      # ===================================================================== #
      when /^-?-?random(-|_)?colou?r$/i,
           /rcolour$/i,
           'RCOL'
        do_use_random_colour
      # ===================================================================== #
      # === turtle --colour=slateblue
      # ===================================================================== #
      when /^-?-?colou?r=(.+)/,
           /^-?-?use(-|_)?this(-|_)?colour=(.+)/ # See: http://rubular.com/r/kH829if9FU
        _ = $1.to_s.dup
        _ = $3.to_s.dup if $3
        set_use_this_colour(_)
      # ===================================================================== #
      # === whip --noclear
      # ===================================================================== #
      when 'dont_clear','dontclear','do_not_use_clear',
           /^-?-?no(-|_)?clear$/i,
           'noc'
        do_not_use_clear
      # ===================================================================== #
      # === exploding_bubbles --delay=5
      # === turtles --delay=12
      # ===================================================================== #
      when /-?-?delay=(\d{1,2})/
        set_delay($1)
      else # else it was not found, so we decide what to do.
        # =================================================================== #
        # Handle Konsole-colours next:
        # =================================================================== #
        if ::Colours.is_this_input_a_html_colour?(i.delete('-'))
          set_use_this_colour(
            i.delete('-')
          )
        end
      end
    end
  end

end; end