#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === AsciiParadise::ClockCounter
#
# This class will show a little ASCII clock counter.
# =========================================================================== #
# require 'ascii_paradise/clock_counter/clock_counter.rb'
# AsciiParadise::ClockCounter.new { :rename_kde_konsole_tab }
# =========================================================================== #
module AsciiParadise

class ClockCounter # === AsciiParadise::ClockCounter

  # ========================================================================= #
  # === SLEEP_FOR_N_SECONDS
  # ========================================================================= #
  SLEEP_FOR_N_SECONDS = 1

  # ========================================================================= #
  # === RUN_ALREADY
  # ========================================================================= #
  RUN_ALREADY = true

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = RUN_ALREADY
    )
    register_sigint
    reset
    case run_already
    when :no_colours
      disable_colours
      run_already = RUN_ALREADY
    end
    if block_given?
      yielded = yield
      case yielded
      # ===================================================================== #
      # === :rename_kde_konsole_tab
      # ===================================================================== #
      when :rename_kde_konsole_tab
        do_rename_kde_konsole_tab
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    use_colours
    set_mode
    # ======================================================================= #
    # === @rename_kde_konsole_tab
    #
    # If the following variable is true then this class will rename the
    # KDE konsole tab as well. By default this is false, but this
    # can be changed by the user.
    # ======================================================================= #
    @rename_kde_konsole_tab = false
  end

  # ========================================================================= #
  # === do_rename_kde_konsole_tab
  # ========================================================================= #
  def do_rename_kde_konsole_tab
    @rename_kde_konsole_tab = true
  end

  # ========================================================================= #
  # === register_sigint
  # ========================================================================= #
  def register_sigint
    trap('INT') { exit_program }
  end

  # ========================================================================= #
  # === set_connected
  # ========================================================================= #
  def set_connected
    set_mode :connected
  end

  # ========================================================================= #
  # === set_mode
  # ========================================================================= #
  def set_mode(i = :standalone)
    case i
    when :standalone
      @run_standalone = true
    else
      @run_standalone = false
    end
  end

  # ========================================================================= #
  # === exit_program
  # ========================================================================= #
  def exit_program
    if @run_standalone == true
      exit
    else
      @run_standalone = :quit
    end
  end

  # ========================================================================= #
  # === disable_colours
  # ========================================================================= #
  def disable_colours
    use_colours(false)
  end

  # ========================================================================= #
  # === use_colours
  # ========================================================================= #
  def use_colours(shall_we_use_colours = true) # We enable colours here.
    begin
      require 'colours'
    rescue LoadError; end
    @use_colours = shall_we_use_colours
  end

  # ========================================================================= #
  # === e
  # ========================================================================= #
  def e(i) # E tag.
    if Object.const_defined? :Colours
      i = Colours::GREEN+i+Colours.rev if @use_colours
    end
    puts i
  end

  # ========================================================================= #
  # === use_colours?
  # ========================================================================= #
  def use_colours?
    @use_colours
  end

  # ========================================================================= #
  # === display_time
  # ========================================================================= #
  def display_time
    use_this_time = Time.now.strftime('%H:%M:%S').strip
    e use_this_time
    # ======================================================================= #
    # Also check if we have to modify the KDE konsole tab.
    # ======================================================================= #
    if @rename_kde_konsole_tab
      Roebe.rename_kde_konsole_tab(use_this_time)
    end
  end

  # ========================================================================= #
  # === consider_requiring_the_kde_konsole_tab_renamer
  # ========================================================================= #
  def consider_requiring_the_kde_konsole_tab_renamer
    if @rename_kde_konsole_tab
      begin
        require 'roebe/requires/require_kde_konsole.rb'
      rescue LoadError; end
    end
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    consider_requiring_the_kde_konsole_tab_renamer
    # ======================================================================= #
    # Our main loop:
    # ======================================================================= #
    loop {
      print `clear`
      display_time
      sleep SLEEP_FOR_N_SECONDS
      break if @run_standalone == :quit
    }
  end

end; end

if __FILE__ == $PROGRAM_NAME
  AsciiParadise::ClockCounter.new # :no_colours # The argument is mostly for testing.
end # clockcounter