#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ascii_paradise/colours/colours.rb'
# =========================================================================== #
require 'ascii_paradise/toplevel_methods/e.rb'
require 'ascii_paradise/constants/misc.rb'

module AsciiParadise

  begin
    require 'colours'
  rescue LoadError; end

  # ========================================================================= #
  # === @colour
  #
  # The toplevel instance variable @colour denotes which colour will be
  # used by default for every animated ASCII file. It is also used by
  # the interactive component of AsciiParadise, to allow the user to
  # quickly specify other colours.
  # ========================================================================= #
  @colour = DEFAULT_COLOUR_TO_USE # Hardcoded value.

  # ========================================================================= #
  # === AsciiParadise.colour?
  # ========================================================================= #
  def self.colour?
    @colour
  end

  # ========================================================================= #
  # === AsciiParadise.set_colour
  # ========================================================================= #
  def self.set_colour(i = nil)
    @colour = i
  end; self.instance_eval { alias set_use_this_colour set_colour } # === AsciiParadise.set_use_this_colour

  # ========================================================================= #
  # === AsciiParadise.clear_colours
  # ========================================================================= #
  def self.clear_colours
    AsciiParadise.set_colour(nil)
  end

  # ========================================================================= #
  # === AsciiParadise.mediumpurple
  # ========================================================================= #
  def self.mediumpurple(i = '')
    Colours.mediumpurple(i)
  end

  # ========================================================================= #
  # === AsciiParadise.simp
  # ========================================================================= #
  def self.simp(i)
    Colours.simp(i)
  end

  # ========================================================================= #
  # === AsciiParadise.sfile
  # ========================================================================= #
  def self.sfile(i)
    Colours.sfile(i)
  end

  # ========================================================================= #
  # === AsciiParadise.sfancy
  # ========================================================================= #
  def self.sfancy(i)
    Colours.sfancy(i)
  end

  # ========================================================================= #
  # === AsciiParadise.swarn
  # ========================================================================= #
  def self.swarn(i)
    Colours.swarn(i)
  end

end