#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ascii_paradise/commandline/help.rb'
# =========================================================================== #
module AsciiParadise

  require 'ascii_paradise/toplevel_methods/e.rb'

  # ========================================================================= #
  # === AsciiParadise.show_help
  #
  # This method can be used to show the available help options in
  # general.
  #
  # It is a class-method primarily so that we can display the help options
  # without having to instantiate a new class.
  #
  # An invocation example for the method follows:
  #
  #   asciispider --help
  #
  # ========================================================================= #
  def self.show_help
    e 'These options are available:'
    e
    array = [] # Store it in an Array, primarily so that we can run .sort on it.
    array << '  --run-n-times=()             # run n times'
    array << '  --no-clear                   # do not invoke "clear"; this '\
             'may be helpful for debugging'
    array << '  --wait-for-keypress          # wait for keypress events'
    array << '  --colour=green               # use a specific colour, in this example green'
    array << '  --colour=random-colour       # this is equivalent to --random-colour'
    array << '  --colourspray                # use a random colour during each (full) '\
             'iteration'
    array << '  --rainbow-line               # this option will colourize every '\
             'individual cell of a given frame'
    array << '  --disco-inferno              # change the colour during every iteration'
    array << '  --half-colour                # use two random colours, one '\
             'for the upper area, another for the lower area '
    array << '  --numbers                    # like --full-debug but also uses red '\
             '"frame numbers"'
    array << '  --random-colour              # use a random colour (useful if '\
             'you do not want to decide on any specific colour)'
    array << '  --overview                   # show the available components '\
             '(most of them at least)'
    array << '  --n_components?              # show how many components are '\
             'available (a number)'
    array << '  --interactive                # enter the interactive part '\
             'of AsciiParadise'
    array << '  --random                     # run a random component of '\
             'the AsciiParadise project'
    array << '  --show-frame=33              # show a specific frame, '\
             'in this case, frame 33'
    array << '  --framerate                  # query, in seconds, the delay '\
             'before the next frame is shown'
    array << '  --delay=2 (Float or Integer) # set the delay to 2 '\
             'seconds, in this case; use any number as input'
    array << '  --random                     # animate a random component'
    array.sort.each {|entry| e entry }
    # ======================================================================= #
    # Check if Konsole is available, as part of the Colours namespace.
    # ======================================================================= #
    unless Object.const_defined? :Colours
      e; e 'Note that you do not appear to have Colours available.'
         e 'This means that we will use normal ansicolours instead.'
    end
    e
  end

end