#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ascii_paradise/commandline/menu.rb'
# =========================================================================== #
module AsciiParadise

  require 'ascii_paradise/toplevel_methods/e.rb'
  require 'ascii_paradise/toplevel_methods/colour_parse_this_string.rb'
  require 'ascii_paradise/toplevel_methods/available_ascii_components.rb'
  require 'ascii_paradise/toplevel_methods/show_available_ascii_components.rb'
  require 'ascii_paradise/toplevel_methods/run_this_animated_component.rb'
  require 'ascii_paradise/colours/colours.rb'
  require 'ascii_paradise/commandline/help.rb'
  require 'ascii_paradise/demo/demo.rb'

  # ========================================================================= #
  # === AsciiParadise.menu                            (menu tag)
  #
  # This method can be used to parse the commandline arguments given to
  # the AsciiParadise project, whenever a user invokes the executable
  # called "ascii_paradise".
  # ========================================================================= #
  def self.menu(
      i = ARGV
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i # case tag
      # ===================================================================== #
      # === ascii_paradise --gtk3
      # ===================================================================== #
      when /^-?-?gtk3?$/i,
           /^-?-?gui$/i
        require 'ascii_paradise/gui/gtk3/animated_frame/widget.rb'
        ::AsciiParadise::GUI::Gtk::Widget.run
      # ===================================================================== #
      # === ascii_paradise --gtk2
      # ===================================================================== #
      when /^-?-?gtk2$/i
        require 'ascii_paradise/gui/gtk2/animated_frame/widget.rb'
        ::AsciiParadise::GUI::Gtk::Widget.run
      # ===================================================================== #
      # === ascii_paradise --pretty-flowers
      # ===================================================================== #
      when /^-?-?pretty(-|_)?flowers$/i,
           /^-?-?flowers$/i,
           /^-?-?beautiful(-|_)?flower$/i
        target = AsciiParadise.static_ascii_directory?+'colourful_flowers.ascii'
        if File.exist? target
          e AsciiParadise.colour_parse_this_string(target)        
        else
          e 'No file exists at '+target+'.'
        end
      # ===================================================================== #
      # === ascii_paradise --show_available_ascii_components
      # ===================================================================== #
      when /^-?-?show(-|_)?available(_|-)?ascii(_|-)?components$/i,
           /^-?-?show(-|_)?available$/i
        show_available_ascii_components
      # ===================================================================== #
      # === ascii_paradise --n-components?
      # ===================================================================== #
      when /^-?-?n(_|-)?components\??$/i
        n_components = Dir["#{::AsciiParadise.animations_directory?}*.rb"].size
        e Colours.rev+'The AsciiParadise project has '+
          sfancy(n_components)+' components available.'
      # ===================================================================== #
      # === ascii_paradise --help
      # ===================================================================== #
      when /^-?-?help$/i
        show_help
      # ===================================================================== #
      # === ascii_paradise --demo
      # ===================================================================== #
      when /^-?-?demo$/i,
           /^-?-?run(_|-)?demo$/i
        run_demo
      # ===================================================================== #
      # === ascii_paradise --colour=slateblue
      # ===================================================================== #
      when /^-?-?colour=(.+)/
        this_colour = $1.to_s.dup
        set_use_this_colour(this_colour)
      # ===================================================================== #
      # === ascii_paradise --available_components
      # ===================================================================== #
      when /^-?-?available(_|-)?components$/,
           /^-?-?show(_|-)?components$/ # === aparadise --show-components
        e
        show_available_ascii_components(
          :show_only_ascii_components
        )
        e
      else
        # =================================================================== #
        # Else assume that the user may have possibly wanted to run some
        # animated component of the AsciiParadise project.
        #
        # Invocation example for this:
        #
        #   aparadise Volcano
        #
        # =================================================================== #
        AsciiParadise.run_this_animated_component(i)
      end
    end
  end; self.instance_eval { alias parse_commandline menu } # === AsciiParadise.parse_commandline

end