#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === AsciiParadise::TableFlip
#
# This code was written by havenwood - ask him for the license.
#
#   https://gist.githubusercontent.com/havenwood/7387731/raw/555d8b0f8d11e5b85921f3d7c55618e8a72fe7b1/table_flip.rb
# =========================================================================== #
# require 'ascii_paradise/curses/table_flip.rb'
# =========================================================================== #
module AsciiParadise

class TableFlip # === AsciiParadise::TableFlip

  begin
    require 'curses'
  rescue LoadError
    # puts 'curses is not available - please install it ' if ENV['IS_ROEBE'].to_s == '1'
  end

  # ========================================================================= #
  # === TableFlip.curses_is_available?
  # ========================================================================= #
  def self.curses_is_available?
    Object.const_defined? :Curses
  end

  # ========================================================================= #
  # === TableFlip.draw
  # ========================================================================= #
  def self.draw(this, fps = 6)
    if curses_is_available?
      Curses.setpos 1, 1
      Curses.addstr this
      Curses.refresh
      sleep(1.0 / fps)
    end
  end

  # ========================================================================= #
  # === TableFlip.register_sigint
  # ========================================================================= #
  def self.register_sigint
    Signal.trap('SIGINT') { exit }
  end

  # ========================================================================= #
  # === TableFlip.run
  # ========================================================================= #
  def self.run
    register_sigint
    loop {
      draw '(*_*)-|--|-'
      draw '(/*_*)-|--|-'
      draw '(/*_*/) -|--|-'
      draw '(/*_*)/  _|__|_'
      draw '(/*_*)/ ~ _|__|_'
      draw '(/*_*)/ ~   _|__|_'
      draw '(*_*)/       _|__|_'
      draw '\(*_*)/      _|__|_'
      draw '/(*_*)/      _|__|_'
      draw '\(*_*)/      _|__|_'
      draw '\(*_*)\      _|__|_'
      draw '  \(*_*)     _|__|_'
      draw '    (*_*)\   _|__|_'
      draw '     (*_*)   _|__|_'
      draw '       (*_*) _|__|_'
      draw '        (*_*)_|__|_'
      draw '             _|__|_(*_*)'
      draw '             _|__|_(*_*\)'
      draw '            -|--|-\(*_*\)'
      draw '        -|--|- ~ \(*_*\) '
      draw '      -|--|-    (*_*\)  '
      draw '     -|--|-   (*_*\)  '
      draw '     -|--|- (*_*\)  '
      draw '     -|--|-(*_*\) '
      draw '     -|--|-(*_*) '
      draw '(*_*)-|--|-     '
    }
  end

end

  # ========================================================================= #
  # === AsciiParadise.table_flip
  # ========================================================================= #
  def self.table_flip
    AsciiParadise::TableFlip.run
  end

  # ========================================================================= #
  # === AsciiParadise.TableFlip
  # ========================================================================= #
  def self.TableFlip
    AsciiParadise::TableFlip
  end

end

if __FILE__ == $PROGRAM_NAME
  AsciiParadise::TableFlip.run
end # table_flip