#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === AsciiParadise::GUI::Gtk::AnimatedFrameModule
# =========================================================================== #
# require 'ascii_paradise/gui/shared_code/animated_frame/animated_frame_module.rb'
# include ::AsciiParadise::GUI::Gtk::AnimatedFrameModule
# =========================================================================== #
module AsciiParadise

module GUI

module Gtk

module AnimatedFrameModule

  require 'gtk_paradise/requires/require_the_base_module.rb'
  include ::Gtk::BaseModule

  require 'ascii_paradise/toplevel_methods/available_ascii_components.rb'
  require 'ascii_paradise/toplevel_methods/return_dataset_of_this_animated_ascii_component.rb'

  # ======================================================================= #
  # === NAMESPACE
  # ======================================================================= #
  NAMESPACE = inspect

  # ======================================================================= #
  # === TITLE
  # ======================================================================= #
  TITLE = 'Animated Frame'

  # ======================================================================= #
  # === WIDTH
  # ======================================================================= #
  WIDTH = 1540

  # ======================================================================= #
  # === HEIGHT
  # ======================================================================= #
  HEIGHT = 720

  # ======================================================================= #
  # === USE_THIS_FONT
  # ======================================================================= #
  USE_THIS_FONT = 'Mono 20'

  # ======================================================================= #
  # === initialize
  # ======================================================================= #
  def initialize(
      run_already = true
    )
    super(:vertical)
    reset
    set_font(USE_THIS_FONT)
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    reset_the_internal_variables
    # ======================================================================= #
    # === @configuration
    # ======================================================================= #
    @configuration = [true, __dir__, NAMESPACE]
    # ======================================================================= #
    # === @title
    # ======================================================================= #
    @title = TITLE
    # ======================================================================= #
    # === @width
    # ======================================================================= #
    set_width(WIDTH)
    # ======================================================================= #
    # === @height
    # ======================================================================= #
    set_height(HEIGHT)
    # ======================================================================= #
    # === @array_dataset
    # ======================================================================= #
    @array_dataset = [] # This variable will contain the whole Array.
    # ======================================================================= #
    # === @available_components
    # ======================================================================= #
    @available_components = ::AsciiParadise.available_components? - [:AnimatedProgressBar]
    append_project_css_file
  end

  # ======================================================================= #
  # === create_skeleton
  #
  # Create the GUI-skeleton via this method.
  # ======================================================================= #
  def create_skeleton
    label_on_top = bold_label('Showing the individual frames')
    minimal(label_on_top, 0)
    @text_buffer = gtk_text_buffer
    @text_view = gtk_text_view(@text_buffer)
    @text_view.set_editable(true)
    @text_view.cursor_visible = false

    @scrolled_window = gtk_scrolled_window(@text_view)
    if @scrolled_window.respond_to? :min_content_height
      @scrolled_window.min_content_height = 500
    end
    @scrolled_window.show_scrollbars

    maximal(@scrolled_window, 2)
    create_the_combo_box_entry
    populate_the_combo_box
    @combo_box_entry.active = 0
    @combo_box_entry.on_changed {
      if @combo_box_entry.active_iter
        active_text = @combo_box_entry.active_text
        @array_dataset = AsciiParadise.dataset_for(
          active_text,
          :do_not_uppercase
        )
        # =================================================================== #
        # We will display the first element of the Array.
        # =================================================================== #
        if @array_dataset.respond_to? :first
          set_text(@array_dataset.first)
        end
      end
    }
    maximal(@combo_box_entry, 0)
    show_all
  end

  # ========================================================================= #
  # === set_text
  #
  # This method can be used to set a new text for the main buffer.
  # ========================================================================= #
  def set_text(i)
    @text_view.buffer.set_text(i)
  end

  # ========================================================================= #
  # === minimum_height?
  # ========================================================================= #
  def minimum_height?
    200
  end

  # ========================================================================= #
  # === minimum_width?
  # ========================================================================= #
  def minimum_width?
    120
  end

  # ========================================================================= #
  # === border_size?
  # ========================================================================= #
  def border_size?
    10
  end

  # ========================================================================= #
  # === AsciiParadise::GUI::Gtk::AnimatedFrameModule.run
  # ========================================================================= #
  def self.run
    require 'gtk_paradise/run'
    r = ::Gtk.run
    r.set_title(
      'Animated Frame Module'
    )
    _ = ::AsciiParadise::GUI::Gtk::AnimatedFrame.new
    r.automatic_title
    r.automatic_size
    r.set_width(_.default_width?)
    r.set_height(_.default_height?)
    r << _
    r.show_all
    r.top_left_then_run
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    create_skeleton
  end

  # ========================================================================= #
  # === populate_the_combo_box
  # ========================================================================= #
  def populate_the_combo_box
    # Populate the combo-box.
    @available_components.each {|entry|
      @combo_box_entry.append_text(entry)
    }
  end

  # ======================================================================= #
  # === create_the_combo_box_entry
  # ======================================================================= #
  def create_the_combo_box_entry
    # ======================================================================= #
    # Add the combobox next.
    # ======================================================================= #
    if ::Gtk.use_gtk2?
      @combo_box_entry = gtk_combo_box_entry(:text_only)
    else
      @combo_box_entry = gtk_combo_box_text
    end
    @combo_box_entry.css_class('combo_box1')
  end

end; end; end; end