#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === AsciiParadise::Lobster
#
# Usage example:
#   AsciiParadise::Lobster.new
# =========================================================================== #
# require 'ascii_paradise/misc/lobster.rb'
# =========================================================================== #
require 'ascii_paradise/base/base.rb'
begin
  require 'zlib'
rescue LoadError; end

module AsciiParadise

class Lobster < Base # === Lobster.new

  # ========================================================================= #
  # === LOBSTER_RIGHT
  # ========================================================================= #
  LOBSTER_RIGHT = Zlib::Inflate.inflate("eJx9kEEOwyAMBO99xd7MAcytUhPlJyj2
  P6jy9i4k9EQyGAnBarEXeCBqSkntNXsi/ZCvC48zGQoZKikGrFMZvgS5ZHd+aGWVuWwhVF0
  t1drVmiR42HcWNz5w3QanT+2gIvTVCiE1lm1Y0eU4JGmIIbaKwextKn8rvW+p5PIwFl8ZWJ
  I8jyiTlhTcYXkekJAzTyYN6E08A+dk8voBkAVTJQ==".delete("\n ").unpack('m*')[0])

  # ========================================================================= #
  # === LOBSTER_LEFT
  # ========================================================================= #
  LOBSTER_LEFT = LOBSTER_RIGHT.split("\n").
                 map { |line| line.ljust(42).reverse }.join("\n")

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    register_sigint
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @delay = 0.75
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    run_n_times = 50
    current_run =  1
    toggle_status = :right
    loop {
      ::AsciiParadise.clear_screen
      case toggle_status
      when :right
        e LOBSTER_RIGHT
        toggle_status = :left
      when :left
        e LOBSTER_LEFT
        toggle_status = :right
      end
      sleep @delay
      current_run += 1
      break if current_run > run_n_times
    }
  end

  # ========================================================================= #
  # === AsciiParadise::Lobster[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end

# =========================================================================== #
# === AsciiParadise.lobster
# =========================================================================== #
def self.lobster(i = ARGV)
  AsciiParadise::Lobster.new(i)
end

end

if __FILE__ == $PROGRAM_NAME
  AsciiParadise::Lobster.new(ARGV)
end # lobster