#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === AsciiParadise::Sparky::CLI
# =========================================================================== #
module AsciiParadise

class Sparky

class CLI

  # ========================================================================= #
  # === AsciiParadise::Sparky::CLI.run
  #
  # Helper method to run an instance with the given arguments.
  #
  # @see #run
  # @return [CLI] the instance that ran.
  # ========================================================================= #
  def self.run(*i)
    instance = new
    instance.run(*i)
    instance
  end

  # ========================================================================= #
  # === run
  #
  # Runs sparky with the given input argument.
  #
  # The method will return nil.
  # ========================================================================= #
  def run(*i)
    if i.empty? || (i.size == 1 && %w(-h --help).include?(i.first))
      puts help
    else
      sparkline = Sparky.new(i.map(&:to_f))
      puts sparkline.to_s
    end
  end

  # ========================================================================= #
  # === help
  #
  # Returns usage information for the sparkyc omponent.
  # ========================================================================= #
  def help
    "
USAGE:
  sparky [-h|--help] VALUE,...

EXAMPLES:
  sparky 1 5 22 13 53
  ▁▁▃▂▇

  sparky 0,30,55,80,33,150
  ▁▂▃▅▂▇

  echo 9 13 5 17 1 | sparky
  ▄▆▂█▁

    "
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  puts AsciiParadise::Sparky::CLI.new.help
end # $ASCII_PARADISE/sparky/cli.rb