#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === AsciiParadise::SierpinksiTriangle
#
# The code in this class is roughly equivalent to the following one-liner:
#
#   ruby -le '64.times { |y| print " " * (63-y),(0..y).map {|x|~y&x>0?" .":" A"}}'
#
# =========================================================================== #
require 'ascii_paradise/base/base.rb'

module AsciiParadise

class SierpinksiTriangle < Base # === AsciiParadise::SierpinksiTriangle

  # ========================================================================= #
  # === N_TIMES
  # ========================================================================= #
  N_TIMES = 30 # This constant will determine how many lines to walk.

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      optional_input = ARGV,
      run_already    = true
    )
    reset
    handle_input(
      optional_input
    )
    run if run_already
  end

  # ========================================================================= #
  # === handle_input
  #
  # This method will handle input given to this class.
  # ========================================================================= #
  def handle_input(i)
    if i
      if i.is_a?(Array) and !i.empty?
        @n_times = i.first
      elsif i.is_a?(Array) and i.empty? # Array is empty, thus use the default.
        @n_times = N_TIMES
      elsif i.is_a? Hash
        if i.has_key? :size
          @n_times = i.delete :size
        end
      else
        @n_times = i
      end
      @n_times = @n_times.to_i
    end
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    @n_times = N_TIMES
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    @n_times.times { |counter|
      joined_substring = (0 .. counter).map {|entry|
        # =================================================================== #
        # Either add a ' .' token or a ' A' token.
        # =================================================================== #
        ~counter & entry > 0 ? ' .' : ' A'
      }.join
      print ' ' * (@n_times-1-counter), joined_substring
      e
    }
  end

end

  # ========================================================================= #
  # === AsciiParadise.sierpinski_triangle
  # ========================================================================= #
  def self.sierpinski_triangle(size = 80)
    AsciiParadise::SierpinksiTriangle.new(size)
  end; self.instance_eval { alias triangle sierpinski_triangle } # === AsciiParadise.triangle

  # ========================================================================= #
  # === AsciiParadise.SierpinskiTriangle
  # ========================================================================= #
  def self.SierpinskiTriangle
    AsciiParadise::SierpinksiTriangle
  end

end

if __FILE__ == $PROGRAM_NAME
  AsciiParadise::SierpinksiTriangle.new
end # sierpinksi_triangle