#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ascii_paradise/toplevel_methods/figlet.rb'
# =========================================================================== #
require 'ascii_paradise/toplevel_methods/e.rb'

module AsciiParadise

  # ========================================================================= #
  # === AsciiParadise.figlet
  #
  # This method requires the program called 'figlet' installed - otherwise
  # it will not work. If you wish to add more fonts then you can download
  # them from websites such as http://www.jave.de/figlet/fonts/overview.html
  # or other sites.
  #
  # Figlet is a text-to-ASCII "symbols" converter. To make use of fonts,
  # you can issue something like this:
  #
  #   figlet -w 120 -f ~/Downloads/broadway.flf
  #
  # The first argument to this method should be a String. This string
  # will be "figletized" - that is, converted into an ASCII representation.
  #
  # The second (optional) argument can be used to specify the path to
  # a locally existing .flf file (font file).
  #
  # Note that this method will return a String.
  #
  # The following list shows some fonts which I consider as
  # useful; many other fonts are quite crappy, though.
  #
  # Invocation examples:
  #
  #   AsciiParadise.figlet 'Wikipedia'
  #   AsciiParadise.figlet 'Wikipedia', '3d_diagonal'
  #   AsciiParadise.figlet 'Wikipedia', :acrobatic
  #   AsciiParadise.figlet 'Wikipedia', :alpha
  #   AsciiParadise.figlet 'Wikipedia', :broadway
  #   AsciiParadise.figlet 'Wikipedia', :colossal
  #   AsciiParadise.figlet 'Wikipedia', :doh
  #   AsciiParadise.figlet 'Wikipedia', 'Georgia11'
  #   AsciiParadise.figlet 'Wikipedia', 'pebbles'
  #   AsciiParadise.figlet 'Wikipedia', 'univers'
  #
  # ========================================================================= #
  def self.figlet(
      i, optional_use_this_font = nil
    )
    if i.is_a? Array
      i = i.join(' ').strip
    end
    cmd_to_use = 'figlet -t'.dup
    if optional_use_this_font
      unless optional_use_this_font.include? '/'
        # =================================================================== #
        # Use a hardcoded path in this case.
        # =================================================================== #
        optional_use_this_font.prepend('/Depot/jjj/fonts/')
      end
      cmd_to_use << ' -f '+optional_use_this_font.to_s
      unless cmd_to_use.end_with? '.flf'
        cmd_to_use << '.flf'
      end
    end
    cmd_to_use << " #{i}"
    result = `#{cmd_to_use}`
    return result
  end

  # ========================================================================= #
  # === AsciiParadise.efiglet
  #
  # This is as figlet(), but it will output the result.
  # ========================================================================= #
  def self.efiglet(i)
    e figlet(i)
  end

end

if __FILE__ == $PROGRAM_NAME
  puts AsciiParadise.figlet(ARGV)
end # ascii_figlet hello world!