#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ascii_paradise/toplevel_methods/return_dataset_of_this_animated_ascii_component.rb'
# dataset = AsciiParadise.dataset_for(:rubber_duck)
# =========================================================================== #
module AsciiParadise

  require 'ascii_paradise/requires/require_animations_files.rb'
  require 'ascii_paradise/toplevel_methods/to_camelcase.rb'

  # ========================================================================= #
  # === AsciiParadise.return_dataset_of_this_animated_ascii_component
  #
  # This method can retrieve the dataset of a given animated ASCII
  # component. This ASCII component must have been registered, though,
  #
  # The argument to this method should be camelcased, e. g. :RubberDuck
  # rather than :rubber_duck. In order to simplify the use of the API,
  # the user can also send :rubber_duck, though.
  #
  # A String can also be passed into this method, such as 'RubberDuck',
  # but it is recommended to use Symbols, if possible, such as
  # :rubber_duck or :RubberDuck.
  # 
  # Usage example:
  #
  #   dataset = AsciiParadise.dataset_for(:rubber_duck)
  #   dataset = AsciiParadise.dataset_for(:table_tennis)
  #   dataset = AsciiParadise.dataset_for('TableTennis')
  #   dataset = AsciiParadise.return_dataset_of_this_animated_ascii_component(:volcano)
  #
  # ========================================================================= #
  def self.return_dataset_of_this_animated_ascii_component(
      i                    = :volcano, # :rubber_duck,
      convert_to_camelcase = true
    )
    case convert_to_camelcase
    when :do_not_uppercase
      convert_to_camelcase = false
    end
    if i.is_a? Array
      i = i.first
    end
    # ======================================================================= #
    # Initialize the animated ASCII components next:
    # ======================================================================= #
    initialize_the_animated_ascii_components

    i = i.to_s
    i = to_camelcase(i) if convert_to_camelcase
    unless AsciiParadise.respond_to? i
      array_available_ascii_components = AsciiParadise.available_ascii_components?
      # ===================================================================== #
      # Try to find a fitting component in this case.
      # ===================================================================== #
      possible_matches = array_available_ascii_components.map {|inner_entry|
        inner_entry.to_s.downcase
      }.select {|entry| entry.include? i.to_s.downcase }
      if !possible_matches.empty?
        # =================================================================== #
        # In this case we have at the least one partial match.
        # =================================================================== #
        i = array_available_ascii_components.select {|entry|
          entry.to_s.downcase.include?(i.to_s.downcase)
        }
        if i.is_a? Array
          i = i.first
        end
      end
    end
    if AsciiParadise.respond_to? i
      instance = AsciiParadise.send(i)
      # ======================================================================= #
      # The dataset that will be returned is an Array.
      # ======================================================================= #
      dataset = instance.new { :do_not_run_yet }.dataset?
      return dataset
    else
      e "Not an animated component (#{i})."
      false
    end
  end; self.instance_eval { alias return_dataset_for  return_dataset_of_this_animated_ascii_component } # === AsciiParadise.return_dataset_for
       self.instance_eval { alias dataset_for         return_dataset_of_this_animated_ascii_component } # === AsciiParadise.dataset_for
       self.instance_eval { alias obtain_dataset_for  return_dataset_of_this_animated_ascii_component } # === AsciiParadise.obtain_dataset_for
       self.instance_eval { alias obtain_dataset_from return_dataset_of_this_animated_ascii_component } # === AsciiParadise.obtain_dataset_from

end

if __FILE__ == $PROGRAM_NAME
  array = AsciiParadise.return_dataset_of_this_animated_ascii_component(ARGV)
  if array
    array.each {|frame| puts frame }
    puts 'This animated ASCII component has '+array.size.to_s+' frames.'
  end
end # returndatasetofthisanimatedasciicomponent tears_of_ice
    # returndatasetofthisanimatedasciicomponent turtle