#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ascii_paradise/toplevel_methods/run_this_animated_component.rb'
# AsciiParadise.run_this_animated_component
# =========================================================================== #
module AsciiParadise

  require 'ascii_paradise/toplevel_methods/available_ascii_components.rb'
  require 'ascii_paradise/toplevel_methods/to_camelcase.rb'
  require 'ascii_paradise/requires/require_animations_files.rb'

  # ========================================================================= #
  # === AsciiParadise.run_this_animated_component
  #
  # This toplevel method can be used to run an animated component of
  # the AsciiParadise project. The main purpose is, primarily, to
  # allow this from the commandline directly, so that it can be
  # used from the file residing at bin/ascii_paradise directly.
  #
  # The API use may be like this:
  #
  #   AsciiParadise.run_this_animated_component '--endless_runner'
  #   AsciiParadise.run_this_animated_component '--chopper'
  #
  # Note that you can also obtain individual frames, like this:
  #
  #   AsciiParadise.run(:spider) { :frame5 }
  #   AsciiParadise.run(:spider) { 5 }
  #
  # This component will be returned.
  # ========================================================================= #
  def self.run_this_animated_component(
      i = :default
    )
    if i.is_a? Array
      i = i.join(' ').strip # Aggregate the Array into a String.
    end
    case i
    when :default
      i = 'endless_runner' # This is the default value.
    end
    require 'abbrev'
    array_available_constants = available_ascii_components?
    this_constant = i.to_s.delete('-') # This will now be something like "endless_runner".
    case this_constant # Use some popular aliases/shortcuts.
    when 'tennis'
      this_constant = 'table_tennis'
    end
    if this_constant.include? '_' # This requires a camelcase-operation!
      this_constant = to_camelcase(this_constant)
    end
    unless array_available_constants.include? this_constant
      # ===================================================================== #
      # In this case, the input was not found, so we will try to
      # abbrev-find something more useful.
      # ===================================================================== #
      flattened = Abbrev.abbrev(array_available_constants).flatten
      alternatives = Hash[*flattened.sort]
      if alternatives.has_key? this_constant
        # =================================================================== #
        # Ok, the hash includes that key, so find the real constant next.
        # =================================================================== #
        this_constant = alternatives[this_constant]
      end
    end
    if this_constant.include? '=' # In this case, assume input such as "colour=slateblue"
      # ======================================================================= #
      # Pass through in this case. Not sure if this is still in use or not,
      # though.
      # ======================================================================= #
    else
      # ======================================================================= #
      # In a constant, the first character is ALWAYS upcased. The following
      # line ensures this to be the case.
      # ======================================================================= #
      this_constant = this_constant.dup if this_constant.frozen?
      this_constant[0,1] = this_constant[0,1].upcase
      # ======================================================================= #
      # Ensure that the animated ASCII components are initialized next.
      # ======================================================================= #
      initialize_the_animated_ascii_components
      if ::AsciiParadise.const_defined? this_constant
        yielded = nil
        if block_given?
          yielded = yield
        end
        if yielded
          if yielded.is_a? Symbol
            # Assume a symbol such as: frame5
            if yielded.to_s.start_with?('frame')
              yielded = yielded.to_s.sub(/^frame/,'').to_i
            end
          end
          if yielded.is_a? Numeric
            _ = AsciiParadise.return_dataset_of_this_animated_ascii_component(this_constant)
            return _[yielded - 1]
          end
        end
        ::AsciiParadise.send(this_constant).animate # We will make use of the .animate method.
      end
    end
  end; self.instance_eval { alias run run_this_animated_component } # === AsciiParadise.run

  # ========================================================================= #
  # === AsciiParadise.run_random_animation
  #
  # This method can be used to run a random animation.
  # ========================================================================= #
  def self.run_random_animation
    _ = available_ascii_components?.sample
    run_this_animated_component(_)
  end; self.instance_eval { alias random           run_random_animation } # === AsciiParadise.random
       self.instance_eval { alias random_component run_random_animation } # === AsciiParadise.random_component

  # ========================================================================= #
  # === AsciiParadise[]
  #
  # This is an alias towards one of the registered methods listed above.
  #
  # If it can not be found, we will use the last else clause and delegate
  # to AsciiParadise.ascii_counter() instead.
  # ========================================================================= #
  def self.[](i = 12)
    _ = available_ascii_components?
    case i
    when 'random','', # empty string also invokes random, as of Nov 2015.
         /^run(_|-)?random(_|-)?animation$/i
      run_random_animation
    else
      if i.to_s =~ /^\d+$/
        run_this_animated_component(
          _[i.to_i - 1]
        )
      else
        run_this_animated_component(i)
      end
    end
  end

end

if __FILE__ == $PROGRAM_NAME
  AsciiParadise.run_this_animated_component(ARGV)
end # run_this_animated_component