#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ascii_paradise/toplevel_methods/show_available_ascii_components.rb'
# AsciiParadise.show_available_ascii_components
# =========================================================================== #
require 'ascii_paradise/toplevel_methods/e.rb'
require 'ascii_paradise/project/project.rb'
require 'ascii_paradise/constants/misc.rb'

module AsciiParadise

  # ========================================================================= #
  # === AsciiParadise.show_available_ascii_components
  #
  # This method will show the available ASCII classes.
  #
  # Since as of March 2017, this class accepts an optional argument,
  # which should be a Symbol. The default Symbol is :everything
  # which means that we will display everything, including the
  # LINKS.md file - on my home system at least.
  #
  # If the argument is :show_only_ascii_components then we will
  # only show the ASCII components of the project.
  #
  # To invoke this, do:
  #
  #   aparadise --help
  #   aparadise --show-available
  #
  # ========================================================================= #
  def self.show_available_ascii_components(
      display_mode = :everything
    )
    target = ::AsciiParadise.animations_directory?
    if display_mode == :everything
      e 'The following classes are part of this project:'
      e
    end
    array = []
    # ======================================================================= #
    # Next, obtain all entries residing in the Animations directory.
    # ======================================================================= #
    Dir["#{target}*.rb"].each {|file|
      # ===================================================================== #
      # Next, we modify it a bit, by getting rid of leading "ascii_" and
      # trailing ".rb" part, then capitalizing it:
      # ===================================================================== #
      _ = File.basename(file).sub(/^ascii_/,'').sub(/\.rb$/,'').capitalize.dup
      # ===================================================================== #
      # Followed by camelcasing it:
      # ===================================================================== #
      if _.include? '_'
        _ = _.split('_').map {|e| e.capitalize }.join
      end
      array << _ # And adding it to our Array. \o/
    }
    # ======================================================================= #
    # Next, display that array in a sorted manner.
    # ======================================================================= #
    array.sort.each_with_index {|entry, _index|
      _index += 1
      case display_mode
      when :everything, :default
        e '  '+sfancy(entry)
      when :show_only_ascii_components
        e simp(_index.to_s.rjust(5))+') '+sfancy(entry)
      end
    }
    case display_mode
    # ======================================================================= #
    # === :everything
    # ======================================================================= #
    when :everything
      e
      # ===================================================================== #
      # === Next, display the LINKS.md file if it is available.
      #
      # This is the primary use case for :everything.
      # ===================================================================== #
      _ = "#{HOME_DIRECTORY_OF_THE_USER_X}DATA/PROGRAMMING_LANGUAGES/RUBY/src/ascii_paradise/doc/LINKS.md"
      if File.exist? _
        e 'Additionally, these links are available:'; e
        File.read(_).split(N).select {|entry| entry.start_with? '  '}.each {|file|
          e '  '+simp(file)
        }; e
      end
    end
  end; self.instance_eval { alias report_what_is_available show_available_ascii_components } # === AsciiParadise.report_what_is_available

  # ========================================================================= #
  # === AsciiParadise.return_what_is_available
  # ========================================================================= #
  def self.return_what_is_available
    target = ::AsciiParadise.animations_directory?
    array = []
    # ======================================================================= #
    # Next, obtain all entries residing in the Animations directory.
    # ======================================================================= #
    Dir["#{target}*.rb"].each {|file|
      # ===================================================================== #
      # Next, we modify it a bit, by getting rid of leading "ascii_" and
      # trailing ".rb" part, then capitalizing it:
      # ===================================================================== #
      _ = File.basename(file).sub(/^ascii_/,'').sub(/\.rb$/,'').capitalize.dup
      # ===================================================================== #
      # Followed by camelcasing it:
      # ===================================================================== #
      if _.include? '_'
        _ = _.split('_').map {|e| e.capitalize }.join
      end
      array << _ # And adding it to our Array. \o/
    }
    return array 
  end

end